@@ -357,13 +357,10 @@ static ssize_t nilfs_get_snapshot(struct nilfs *nilfs, nilfs_cno_t **ssp)
 		for (j = 0; j < n; j++) {
 			ss[i + j] = cpinfo[j].ci_cno;
 			if (prev >= ss[i + j]) {
-				nilfs_gc_logger
-					(LOG_ERR,
-					 "broken snapshot information. "
-					 "snapshot numbers appeared in a "
-					 "non-ascending order: %llu >= %llu",
-					 (unsigned long long)prev,
-					 (unsigned long long)ss[i + j]);
+				nilfs_gc_logger(LOG_ERR,
+						"broken snapshot information. snapshot numbers appeared in a non-ascending order: %llu >= %llu",
+						(unsigned long long)prev,
+						(unsigned long long)ss[i + j]);
 				free(ss);
 				errno = EIO;
 				return -1;
@@ -508,8 +505,7 @@ static int nilfs_toss_vdescs(struct nilfs *nilfs,
 	}
 	ret = 0;
  out:
-	if (ss != NULL)
-		free(ss);
+	free(ss);
 	return ret;
 }
 
@@ -802,7 +798,7 @@ out_lock:
 	if (nilfs_unlock_cleaner(nilfs) < 0) {
 		nilfs_gc_logger(LOG_CRIT, "failed to unlock cleaner: %s",
 				strerror(errno));
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 
 out_sig:
