@@ -13,11 +13,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * Written by Koji Sato <koji@osrg.net>.
+ * Written by Koji Sato.
  *
  * Maintained by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp> from 2008.
  */
@@ -381,6 +377,7 @@ struct nilfs *nilfs_open(const char *dev, const char *dir, int flags)
 	nilfs->n_iocfd = -1;
 	nilfs->n_dev = NULL;
 	nilfs->n_ioc = NULL;
+	nilfs->n_opts = 0;
 	nilfs->n_mincno = NILFS_CNO_MIN;
 	memset(nilfs->n_sems, 0, sizeof(nilfs->n_sems));
 
@@ -857,19 +854,17 @@ int nilfs_put_segment(struct nilfs *nilfs, void *segment)
 		return -1;
 	}
 
-#ifdef HAVE_MUNMAP
+#ifdef HAVE_MMAP
 	if (nilfs_opt_test_mmap(nilfs)) {
 		segsize = le32_to_cpu(nilfs->n_sb->s_blocks_per_segment) *
 			(1UL << (le32_to_cpu(nilfs->n_sb->s_log_block_size) +
 				 NILFS_SB_BLOCK_SIZE_SHIFT));
 		return munmap(segment, segsize);
-	} else {
-#endif	/* HAVE_MUNMAP */
-		free(segment);
-		return 0;
-#ifdef HAVE_MUNMAP
 	}
-#endif	/* HAVE_MUNMAP */
+#endif	/* HAVE_MMAP */
+
+	free(segment);
+	return 0;
 }
 
 __u64 nilfs_get_segment_seqnum(const struct nilfs *nilfs, void *segment,
@@ -892,17 +887,34 @@ __u64 nilfs_get_segment_seqnum(const struct nilfs *nilfs, void *segment,
 /* nilfs_psegment */
 static int nilfs_psegment_is_valid(const struct nilfs_psegment *pseg)
 {
-	int offset;
+	__u32 restblocks, sumbytes, offset;
 
 	if (le32_to_cpu(pseg->p_segsum->ss_magic) != NILFS_SEGSUM_MAGIC)
 		return 0;
 
 	offset = sizeof(pseg->p_segsum->ss_datasum) +
 		sizeof(pseg->p_segsum->ss_sumsum);
-	return le32_to_cpu(pseg->p_segsum->ss_sumsum) ==
-		crc32_le(pseg->p_seed,
-			 (unsigned char *)pseg->p_segsum + offset,
-			 le32_to_cpu(pseg->p_segsum->ss_sumbytes) - offset);
+	restblocks = pseg->p_segblocknr + pseg->p_maxblocks - pseg->p_blocknr;
+	sumbytes = le32_to_cpu(pseg->p_segsum->ss_sumbytes);
+
+	if (sumbytes < offset ||
+	    sumbytes > (__u64)restblocks * pseg->p_blksize)
+		return 0;
+
+	if (le32_to_cpu(pseg->p_segsum->ss_sumsum) !=
+	    crc32_le(pseg->p_seed,
+		     (unsigned char *)pseg->p_segsum + offset,
+		     sumbytes - offset))
+		return 0;
+
+	/*
+	 * Sanity check on segment summary information - violation
+	 * below should be handled as an error in a future release.
+	 */
+	if (le16_to_cpu(pseg->p_segsum->ss_bytes) > sumbytes)
+		return 0;
+
+	return 1;
 }
 
 void nilfs_psegment_init(struct nilfs_psegment *pseg, __u64 segnum,
@@ -910,9 +922,10 @@ void nilfs_psegment_init(struct nilfs_psegment *pseg, __u64 segnum,
 {
 	unsigned long blkoff, nblocks_per_segment;
 
-	blkoff = (segnum == 0) ?
-		le64_to_cpu(nilfs->n_sb->s_first_data_block) : 0;
 	nblocks_per_segment = le32_to_cpu(nilfs->n_sb->s_blocks_per_segment);
+	blkoff = (segnum == 0 ?
+		  min_t(__u64, le64_to_cpu(nilfs->n_sb->s_first_data_block),
+			nblocks_per_segment) : 0);
 
 	pseg->p_blksize = 1UL << (le32_to_cpu(nilfs->n_sb->s_log_block_size) +
 				  NILFS_SB_BLOCK_SIZE_SHIFT);
@@ -928,8 +941,8 @@ void nilfs_psegment_init(struct nilfs_psegment *pseg, __u64 segnum,
 int nilfs_psegment_is_end(const struct nilfs_psegment *pseg)
 {
 	return pseg->p_blocknr >= pseg->p_segblocknr + pseg->p_nblocks ||
-		pseg->p_segblocknr + pseg->p_maxblocks - pseg->p_blocknr <
-		NILFS_PSEG_MIN_BLOCKS ||
+		pseg->p_segblocknr + pseg->p_maxblocks <
+		pseg->p_blocknr + NILFS_PSEG_MIN_BLOCKS ||
 		!nilfs_psegment_is_valid(pseg);
 }
 
@@ -943,6 +956,45 @@ void nilfs_psegment_next(struct nilfs_psegment *pseg)
 }
 
 /* nilfs_file */
+static size_t nilfs_binfo_total_size(unsigned long offset, size_t blksize,
+				     unsigned int binfosize, size_t blkcnt)
+{
+	size_t binfo_per_block, rest = blksize - offset % blksize;
+
+	if (binfosize * blkcnt <= rest)
+		return binfosize * blkcnt;
+
+	blkcnt -= rest / binfosize;
+	binfo_per_block = blksize / binfosize;
+	return rest + (blkcnt / binfo_per_block) * blksize +
+		(blkcnt % binfo_per_block) * binfosize;
+}
+
+static size_t nilfs_file_info_size(const struct nilfs_file *file)
+{
+	unsigned int dsize, nsize;
+	__u32 nblocks, ndatablk;
+	size_t delta, blksize = file->f_psegment->p_blksize;
+
+	if (!nilfs_file_is_super(file)) {
+		dsize = NILFS_BINFO_DATA_SIZE;
+		nsize = NILFS_BINFO_NODE_SIZE;
+	} else {
+		dsize = NILFS_BINFO_DAT_DATA_SIZE;
+		nsize = NILFS_BINFO_DAT_NODE_SIZE;
+	}
+
+	nblocks = le32_to_cpu(file->f_finfo->fi_nblocks);
+	ndatablk = le32_to_cpu(file->f_finfo->fi_ndatablk);
+
+	delta = sizeof(struct nilfs_finfo);
+	delta += nilfs_binfo_total_size(file->f_offset + delta,
+					blksize, dsize, ndatablk);
+	delta += nilfs_binfo_total_size(file->f_offset + delta,
+					blksize, nsize, nblocks - ndatablk);
+	return delta;
+}
+
 void nilfs_file_init(struct nilfs_file *file,
 		     const struct nilfs_psegment *pseg)
 {
@@ -966,55 +1018,51 @@ void nilfs_file_init(struct nilfs_file *file,
 	}
 }
 
+static int nilfs_file_is_valid(const struct nilfs_file *file)
+{
+	const struct nilfs_psegment *pseg = file->f_psegment;
+	__u32 sumbytes = le32_to_cpu(pseg->p_segsum->ss_sumbytes);
+	__u32 nblocks, blkoff;
+
+	/*
+	 * Sanity checks on file summary information.  The violations
+	 * below should be handled as errors in a future release.
+	 */
+	if (file->f_offset + sizeof(struct nilfs_finfo) > sumbytes)
+		return 0;
+
+	nblocks = le32_to_cpu(file->f_finfo->fi_nblocks);
+	blkoff = file->f_blocknr - pseg->p_blocknr;
+	if (blkoff + nblocks > le32_to_cpu(pseg->p_segsum->ss_nblocks))
+		return 0;
+
+	if (le32_to_cpu(file->f_finfo->fi_ndatablk) > nblocks)
+		return 0;
+
+	if (file->f_offset + nilfs_file_info_size(file) > sumbytes)
+		return 0;
+
+	return 1;
+}
+
 int nilfs_file_is_end(const struct nilfs_file *file)
 {
 	return file->f_index >=
-		le32_to_cpu(file->f_psegment->p_segsum->ss_nfinfo);
-}
-
-static size_t nilfs_binfo_total_size(unsigned long offset,
-				     size_t blksize, size_t bisize, size_t n)
-{
-	size_t binfo_per_block, rest = blksize - offset % blksize;
-
-	if (bisize * n <= rest)
-		return bisize * n;
-
-	n -= rest / bisize;
-	binfo_per_block = blksize / bisize;
-	return rest + (n / binfo_per_block) * blksize +
-		(n % binfo_per_block) * bisize;
+		le32_to_cpu(file->f_psegment->p_segsum->ss_nfinfo) ||
+		!nilfs_file_is_valid(file);
 }
 
 void nilfs_file_next(struct nilfs_file *file)
 {
 	size_t blksize, rest, delta;
-	size_t dsize, nsize;
-	unsigned long ndatablk, nblocks;
 
-	blksize = file->f_psegment->p_blksize;
+	delta = nilfs_file_info_size(file);
 
-	if (!nilfs_file_is_super(file)) {
-		dsize = NILFS_BINFO_DATA_SIZE;
-		nsize = NILFS_BINFO_NODE_SIZE;
-	} else {
-		dsize = NILFS_BINFO_DAT_DATA_SIZE;
-		nsize = NILFS_BINFO_DAT_NODE_SIZE;
-	}
-
-	nblocks = le32_to_cpu(file->f_finfo->fi_nblocks);
-	ndatablk = le32_to_cpu(file->f_finfo->fi_ndatablk);
-
-	delta = sizeof(struct nilfs_finfo);
-	delta += nilfs_binfo_total_size(file->f_offset + delta,
-					blksize, dsize, ndatablk);
-	delta += nilfs_binfo_total_size(file->f_offset + delta,
-					blksize, nsize, nblocks - ndatablk);
-
-	file->f_blocknr += nblocks;
+	file->f_blocknr += le32_to_cpu(file->f_finfo->fi_nblocks);
 	file->f_offset += delta;
 	file->f_finfo = (void *)file->f_finfo + delta;
 
+	blksize = file->f_psegment->p_blksize;
 	rest = blksize - file->f_offset % blksize;
 	if (sizeof(struct nilfs_finfo) > rest) {
 		file->f_offset += rest;
