@@ -13,10 +13,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
  * Written by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp>.
  */
 
@@ -190,8 +186,10 @@ int nilfs_sb_write(int devfd, struct nilfs_super_block *sbp, int mask)
 
 	assert(devfd >= 0);
 
-	if (sbp == NULL)
+	if (sbp == NULL) {
+		errno = EINVAL;
 		return -1;
+	}
 
 	if (__nilfs_sb_read(devfd, sbps, offsets))
 		return -1;
