@@ -15,11 +15,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with NILFS; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * Written by Koji Sato <koji@osrg.net>.
+ * Written by Koji Sato.
  *
  * Maintained by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp> from 2008.
  */
@@ -27,21 +23,12 @@
 #ifndef CLDCONFIG_H
 #define CLDCONFIG_H
 
-#include <sys/time.h>
+#if HAVE_TIME_H
+#include <time.h>	/* timespec */
+#endif	/* HAVE_TIME_H */
+
 #include <syslog.h>
 
-struct nilfs_suinfo;
-
-/**
- * struct nilfs_selection_policy -
- * @p_importance:
- * @p_threshold:
- */
-struct nilfs_selection_policy {
-	unsigned long long (*p_importance)(const struct nilfs_suinfo *);
-	unsigned long long p_threshold;
-};
-
 /**
  * struct nilfs_param - parameter with unit suffix
  * @num: scanned value
@@ -94,16 +81,16 @@ enum nilfs_size_unit {
  * if clean segments < min_clean_segments
  */
 struct nilfs_cldconfig {
-	struct nilfs_selection_policy cf_selection_policy;
-	struct timeval cf_protection_period;
+	int cf_selection_policy;
+	struct timespec cf_protection_period;
 	__u64 cf_min_clean_segments;
 	__u64 cf_max_clean_segments;
-	struct timeval cf_clean_check_interval;
+	struct timespec cf_clean_check_interval;
 	int cf_nsegments_per_clean;
 	int cf_mc_nsegments_per_clean;
-	struct timeval cf_cleaning_interval;
-	struct timeval cf_mc_cleaning_interval;
-	struct timeval cf_retry_interval;
+	struct timespec cf_cleaning_interval;
+	struct timespec cf_mc_cleaning_interval;
+	struct timespec cf_retry_interval;
 	int cf_use_mmap;
 	int cf_use_set_suinfo;
 	int cf_log_priority;
@@ -111,9 +98,11 @@ struct nilfs_cldconfig {
 	unsigned long cf_mc_min_reclaimable_blocks;
 };
 
-#define NILFS_CLDCONFIG_SELECTION_POLICY_IMPORTANCE	\
-			nilfs_cldconfig_selection_policy_timestamp
-#define NILFS_CLDCONFIG_SELECTION_POLICY_THRESHOLD	0
+enum nilfs_selection_policy {
+	NILFS_SELECTION_POLICY_TIMESTAMP = 0,
+	__NR_NILFS_SELECTION_POLICY
+};
+
 #define NILFS_CLDCONFIG_PROTECTION_PERIOD		3600
 #define NILFS_CLDCONFIG_MIN_CLEAN_SEGMENTS		10
 #define NILFS_CLDCONFIG_MIN_CLEAN_SEGMENTS_UNIT		NILFS_SIZE_UNIT_PERCENT
