@@ -15,11 +15,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with NILFS; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * Written by Koji Sato <koji@osrg.net>,
+ * Written by Koji Sato,
  *            Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp>.
  */
 
@@ -37,6 +33,10 @@
 #include <unistd.h>
 #endif	/* HAVE_UNISTD_H */
 
+#if HAVE_ERR_H
+#include <err.h>
+#endif	/* HAVE_ERR_H */
+
 #if HAVE_STRING_H
 #include <string.h>
 #endif	/* HAVE_STRING_H */
@@ -69,19 +69,24 @@
 #include <mqueue.h>
 #endif	/* HAVE_MQUEUE_H */
 
+#if HAVE_POLL_H
+#include <poll.h>
+#endif	/* HAVE_POLL_H */
+
 #include <errno.h>
 #include <signal.h>
 #include <setjmp.h>
 #include <assert.h>
 #include <uuid/uuid.h>
 #include "nilfs.h"
+#include "compat.h"
 #include "util.h"
 #include "vector.h"
 #include "nilfs_gc.h"
 #include "nilfs_cleaner.h"
 #include "cleaner_msg.h"
 #include "cldconfig.h"
-#include "cnoconv.h"
+#include "cnormap.h"
 #include "realpath.h"
 
 
@@ -118,7 +123,7 @@ static const struct option long_option[] = {
 /**
  * struct nilfs_cleanerd - nilfs cleaner daemon
  * @nilfs: nilfs object
- * @cnoconv: checkpoint number converter
+ * @cnormap: checkpoint number reverse mapper
  * @config: config structure
  * @conffile: configuration file name
  * @running: running state
@@ -127,7 +132,7 @@ static const struct option long_option[] = {
  * @no_timeout: the next timeout will be 0 seconds
  * @ncleansegs: number of segments cleaned per cycle
  * @cleaning_interval: cleaning interval
- * @target: target time for sleeping
+ * @target: target time for sleeping (monotonic time)
  * @timeout: timeout value for sleeping
  * @min_reclaimable_blocks: min. number of reclaimable blocks
  * @prev_nongc_ctime: previous nongc ctime
@@ -147,7 +152,7 @@ static const struct option long_option[] = {
  */
 struct nilfs_cleanerd {
 	struct nilfs *nilfs;
-	struct nilfs_cnoconv *cnoconv;
+	struct nilfs_cnormap *cnormap;
 	struct nilfs_cldconfig config;
 	char *conffile;
 	int running;
@@ -156,9 +161,9 @@ struct nilfs_cleanerd {
 	int no_timeout;
 	int shutdown;
 	long ncleansegs;
-	struct timeval cleaning_interval;
-	struct timeval target;
-	struct timeval timeout;
+	struct timespec cleaning_interval;
+	struct timespec target;
+	struct timespec timeout;
 	unsigned long min_reclaimable_blocks;
 	__u64 prev_nongc_ctime;
 	mqd_t recvq;
@@ -170,8 +175,8 @@ struct nilfs_cleanerd {
 	int mm_nrestpasses;
 	long mm_nrestsegs;
 	long mm_ncleansegs;
-	struct timeval mm_protection_period;
-	struct timeval mm_cleaning_interval;
+	struct timespec mm_protection_period;
+	struct timespec mm_cleaning_interval;
 	unsigned long mm_min_reclaimable_blocks;
 };
 
@@ -182,11 +187,11 @@ struct nilfs_cleanerd {
  */
 struct nilfs_segimp {
 	__u64 si_segnum;
-	unsigned long long si_importance;
+	long long si_importance;
 };
 
 /* command line option value */
-static volatile unsigned long protection_period;
+static unsigned long protection_period;
 
 /* global variables */
 static struct nilfs_cleanerd *nilfs_cleanerd;
@@ -225,18 +230,20 @@ static void nilfs_cleanerd_set_log_priority(struct nilfs_cleanerd *cleanerd)
 static int nilfs_cleanerd_config(struct nilfs_cleanerd *cleanerd,
 				 const char *conffile)
 {
-	if (nilfs_cldconfig_read(&cleanerd->config,
-				 conffile ? : cleanerd->conffile,
+	struct nilfs_cldconfig *config = &cleanerd->config;
+
+	if (nilfs_cldconfig_read(config, conffile ? : cleanerd->conffile,
 				 cleanerd->nilfs) < 0)
 		return -1;
+
 #ifdef HAVE_MMAP
-	if (cleanerd->config.cf_use_mmap)
+	if (config->cf_use_mmap)
 		nilfs_opt_set_mmap(cleanerd->nilfs);
 	else
 		nilfs_opt_clear_mmap(cleanerd->nilfs);
 #endif	/* HAVE_MMAP */
 
-	if (cleanerd->config.cf_use_set_suinfo)
+	if (config->cf_use_set_suinfo)
 		nilfs_opt_set_set_suinfo(cleanerd->nilfs);
 	else
 		nilfs_opt_clear_set_suinfo(cleanerd->nilfs);
@@ -246,8 +253,8 @@ static int nilfs_cleanerd_config(struct nilfs_cleanerd *cleanerd,
 	if (protection_period != ULONG_MAX) {
 		syslog(LOG_INFO, "override protection period to %lu",
 		       protection_period);
-		cleanerd->config.cf_protection_period.tv_sec = protection_period;
-		cleanerd->config.cf_protection_period.tv_usec = 0;
+		config->cf_protection_period.tv_sec = protection_period;
+		config->cf_protection_period.tv_nsec = 0;
 	}
 	return 0;
 }
@@ -316,7 +323,8 @@ static int nilfs_cleanerd_open_queue(struct nilfs_cleanerd *cleanerd,
 	if (!cleanerd->recvq_name)
 		goto failed;
 
-	cleanerd->recvq = mq_open(nambuf, O_RDONLY | O_CREAT, 0600, &attr);
+	cleanerd->recvq = mq_open(nambuf, O_RDONLY | O_CREAT | O_NONBLOCK,
+				  0600, &attr);
 	if (cleanerd->recvq < 0) {
 		free(cleanerd->recvq_name);
 		goto failed;
@@ -350,6 +358,15 @@ static void nilfs_cleanerd_close_queue(struct nilfs_cleanerd *cleanerd)
 #define PATH_MAX	8192
 #endif	/* PATH_MAX */
 
+static __attribute__((noinline)) char *get_canonical_path(const char *path)
+{
+	char buf[PATH_MAX + 2], *canonical = NULL;
+
+	if (path && myrealpath(path, buf, sizeof(buf)))
+		canonical = strdup(buf);
+	return canonical;
+}
+
 /**
  * nilfs_cleanerd_create - create cleanerd object
  * @dev: name of the device on which the cleanerd operates
@@ -374,16 +391,16 @@ nilfs_cleanerd_create(const char *dev, const char *dir, const char *conffile)
 		goto out_cleanerd;
 	}
 
-	cleanerd->cnoconv = nilfs_cnoconv_create(cleanerd->nilfs);
-	if (cleanerd->cnoconv == NULL) {
+	cleanerd->cnormap = nilfs_cnormap_create(cleanerd->nilfs);
+	if (cleanerd->cnormap == NULL) {
 		syslog(LOG_ERR,
-		       "failed to create checkpoint number converter: %m");
+		       "failed to create checkpoint number reverse mapper: %m");
 		goto out_nilfs;
 	}
 
 	cleanerd->conffile = strdup(conffile ? : NILFS_CLEANERD_CONFFILE);
 	if (cleanerd->conffile == NULL)
-		goto out_cnoconv;
+		goto out_cnormap;
 
 	if (nilfs_cleanerd_config(cleanerd, NULL) < 0)
 		goto out_conffile;
@@ -398,8 +415,8 @@ nilfs_cleanerd_create(const char *dev, const char *dir, const char *conffile)
 	/* error */
 out_conffile:
 	free(cleanerd->conffile);
-out_cnoconv:
-	nilfs_cnoconv_destroy(cleanerd->cnoconv);
+out_cnormap:
+	nilfs_cnormap_destroy(cleanerd->cnormap);
 out_nilfs:
 	nilfs_close(cleanerd->nilfs);
 
@@ -412,7 +429,7 @@ static void nilfs_cleanerd_destroy(struct nilfs_cleanerd *cleanerd)
 {
 	nilfs_cleanerd_close_queue(cleanerd);
 	free(cleanerd->conffile);
-	nilfs_cnoconv_destroy(cleanerd->cnoconv);
+	nilfs_cnormap_destroy(cleanerd->cnormap);
 	nilfs_close(cleanerd->nilfs);
 	free(cleanerd);
 }
@@ -440,7 +457,7 @@ static long nilfs_cleanerd_ncleansegs(struct nilfs_cleanerd *cleanerd)
 		cleanerd->mm_ncleansegs : cleanerd->ncleansegs;
 }
 
-static struct timeval *
+static struct timespec *
 nilfs_cleanerd_cleaning_interval(struct nilfs_cleanerd *cleanerd)
 {
 	return cleanerd->running == 2 ?
@@ -448,7 +465,7 @@ nilfs_cleanerd_cleaning_interval(struct nilfs_cleanerd *cleanerd)
 		&cleanerd->cleaning_interval;
 }
 
-static struct timeval *
+static struct timespec *
 nilfs_cleanerd_protection_period(struct nilfs_cleanerd *cleanerd)
 {
 	return cleanerd->running == 2 ?
@@ -562,49 +579,52 @@ static int nilfs_shrink_protected_region(struct nilfs *nilfs)
  * @oldestp: place to store the oldest mod-time
  */
 #define NILFS_CLEANERD_NSUINFO	512
-#define NILFS_CLEANERD_NULLTIME (~(__u64)0)
+#define NILFS_CLEANERD_NULLTIME S64_MAX
 
 static ssize_t
 nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 			       struct nilfs_sustat *sustat, __u64 *segnums,
-			       __u64 *prottimep, __u64 *oldestp)
+			       __s64 *prottimep, __s64 *oldestp)
 {
 	struct nilfs *nilfs;
-	struct nilfs_cldconfig *config;
 	struct nilfs_vector *smv;
 	struct nilfs_segimp *sm;
 	struct nilfs_suinfo si[NILFS_CLEANERD_NSUINFO];
-	struct timeval tv, tv2;
-	__u64 prottime, oldest;
+	struct timespec ts, ts2;
+	__s64 prottime, oldest, lastmod, now;
 	__u64 segnum;
 	size_t count, nsegs;
 	ssize_t nssegs, n;
-	unsigned long long imp, thr;
+	long long imp, thr;
+	int ret;
 	int i;
 
 	nsegs = nilfs_cleanerd_ncleansegs(cleanerd);
 	nilfs = cleanerd->nilfs;
-	config = &cleanerd->config;
 
 	smv = nilfs_vector_create(sizeof(struct nilfs_segimp));
 	if (!smv)
 		return -1;
 
-	/* The segments that were more recently written to disk than
-	 * prottime are not selected. */
-	if (gettimeofday(&tv, NULL) < 0) {
+	/*
+	 * The segments that were more recently written to disk than
+	 * prottime are not selected.
+	 */
+	ret = clock_gettime(CLOCK_REALTIME, &ts);
+	if (ret < 0) {
 		nssegs = -1;
 		goto out;
 	}
-	timersub(&tv, nilfs_cleanerd_protection_period(cleanerd), &tv2);
-	prottime = tv2.tv_sec;
-	oldest = tv.tv_sec;
+	timespecsub(&ts, nilfs_cleanerd_protection_period(cleanerd), &ts2);
+	now = ts.tv_sec;
+	prottime = ts2.tv_sec;
+	oldest = NILFS_CLEANERD_NULLTIME;
 
-	/* The segments that have larger importance than thr are not
-	 * selected. */
-	thr = (config->cf_selection_policy.p_threshold != 0) ?
-		config->cf_selection_policy.p_threshold :
-		sustat->ss_nongc_ctime;
+	/*
+	 * The segments that have larger importance than thr are not
+	 * selected.
+	 */
+	thr = sustat->ss_nongc_ctime;
 
 	for (segnum = 0; segnum < sustat->ss_nsegs; segnum += n) {
 		count = min_t(__u64, sustat->ss_nsegs - segnum,
@@ -618,11 +638,23 @@ nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 			if (!nilfs_suinfo_reclaimable(&si[i]))
 				continue;
 
-			imp = config->cf_selection_policy.p_importance(&si[i]);
+			/*
+			 * Use local variable 'lastmod' to treat the
+			 * segment timestamp as a signed type value.
+			 */
+			lastmod = si[i].sui_lastmod;
+
+			/*
+			 * Timestamp policy.  The importance value is
+			 * adjusted to include segments with a future
+			 * timestamp.
+			 */
+			imp = lastmod <= now ? lastmod : thr - 1;
+
 			if (imp < thr) {
-				if (si[i].sui_lastmod < oldest)
-					oldest = si[i].sui_lastmod;
-				if (si[i].sui_lastmod < prottime) {
+				if (lastmod < oldest)
+					oldest = lastmod;
+				if (lastmod < prottime || lastmod > now) {
 					sm = nilfs_vector_get_new_element(smv);
 					if (sm == NULL) {
 						nssegs = -1;
@@ -635,8 +667,7 @@ nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 		}
 		if (n == 0) {
 			syslog(LOG_WARNING,
-			       "inconsistent number of segments: %llu "
-			       "(nsegs=%llu)",
+			       "inconsistent number of segments: %llu (nsegs=%llu)",
 			       (unsigned long long)nilfs_vector_get_size(smv),
 			       (unsigned long long)sustat->ss_nsegs);
 			break;
@@ -644,15 +675,14 @@ nilfs_cleanerd_select_segments(struct nilfs_cleanerd *cleanerd,
 	}
 	nilfs_vector_sort(smv, nilfs_comp_segimp);
 
-	nssegs = (nilfs_vector_get_size(smv) < nsegs) ?
-		nilfs_vector_get_size(smv) : nsegs;
+	nssegs = min_t(size_t, nilfs_vector_get_size(smv), nsegs);
 	for (i = 0; i < nssegs; i++) {
 		sm = nilfs_vector_get_element(smv, i);
 		assert(sm != NULL);
 		segnums[i] = sm->si_segnum;
 	}
 	*prottimep = prottime;
-	*oldestp = oldest < tv.tv_sec ? oldest : NILFS_CLEANERD_NULLTIME;
+	*oldestp = oldest;
 
  out:
 	nilfs_vector_destroy(smv);
@@ -677,17 +707,17 @@ static int oom_adjust(void)
 
 	fd = open(path, O_WRONLY);
 	if (fd < 0) {
-		fprintf(stderr, "can't adjust oom-killer's pardon %s, %m\n",
-			path);
-		return errno;
+		syslog(LOG_WARNING,
+		       "can't adjust oom-killer's pardon %s: %m", path);
+		return -1;
 	}
 
 	err = write(fd, score, strlen(score));
 	if (err < 0) {
-		fprintf(stderr, "can't adjust oom-killer's pardon %s, %m\n",
-			path);
+		syslog(LOG_WARNING,
+		       "can't adjust oom-killer's pardon %s: %m", path);
 		close(fd);
-		return errno;
+		return -1;
 	}
 	close(fd);
 	return 0;
@@ -705,7 +735,7 @@ static int daemonize(int nochdir, int noclose)
 		return -1;
 	else if (pid != 0)
 		/* parent */
-		_exit(0);
+		_exit(EXIT_SUCCESS);
 
 	/* child */
 	if (setsid() < 0)
@@ -719,7 +749,7 @@ static int daemonize(int nochdir, int noclose)
 		return -1;
 	else if (pid != 0)
 		/* parent */
-		_exit(0);
+		_exit(EXIT_SUCCESS);
 
 	if (!nochdir && (chdir(ROOTDIR) < 0))
 		return -1;
@@ -755,9 +785,9 @@ static int set_sigterm_handler(void)
 {
 	struct sigaction act;
 
+	memset(&act, 0, sizeof(act));
 	act.sa_handler = handle_sigterm;
 	sigfillset(&act.sa_mask);
-	act.sa_flags = 0;
 	return sigaction(SIGTERM, &act, NULL);
 }
 
@@ -765,9 +795,9 @@ static int set_sighup_handler(void)
 {
 	struct sigaction act;
 
+	memset(&act, 0, sizeof(act));
 	act.sa_handler = handle_sighup;
 	sigfillset(&act.sa_mask);
-	act.sa_flags = 0;
 	return sigaction(SIGHUP, &act, NULL);
 }
 
@@ -822,29 +852,28 @@ static void nilfs_cleanerd_manual_stop(struct nilfs_cleanerd *cleanerd)
 
 static int nilfs_cleanerd_init_interval(struct nilfs_cleanerd *cleanerd)
 {
-	if (gettimeofday(&cleanerd->target, NULL) < 0) {
-		syslog(LOG_ERR, "cannot get time: %m");
+	int ret;
+
+	ret = clock_gettime(CLOCK_MONOTONIC, &cleanerd->target);
+	if (ret < 0) {
+		syslog(LOG_ERR, "cannot get monotonic time: %m");
 		return -1;
 	}
-	timeradd(&cleanerd->target, &cleanerd->config.cf_cleaning_interval,
-		 &cleanerd->target);
+	timespecadd(&cleanerd->target, &cleanerd->config.cf_cleaning_interval,
+		    &cleanerd->target);
 	return 0;
 }
 
 static int nilfs_cleanerd_recalc_interval(struct nilfs_cleanerd *cleanerd,
 					  int nchosen, int ndone,
-					  __u64 prottime, __u64 oldest)
+					  __s64 prottime, __s64 oldest)
 {
-	struct timeval curr, *interval;
-
-	if (gettimeofday(&curr, NULL) < 0) {
-		syslog(LOG_ERR, "cannot get current time: %m");
-		return -1;
-	}
+	struct timespec curr, *interval;
+	int ret;
 
 	if (nchosen == 0 ||
 	    (!cleanerd->fallback && ndone == 0 && !cleanerd->retry_cleaning)) {
-		struct timeval pt;
+		struct timespec pt;
 
 		/* no segment were cleaned */
 		if (cleanerd->running == 2) {
@@ -858,11 +887,19 @@ static int nilfs_cleanerd_recalc_interval(struct nilfs_cleanerd *cleanerd,
 
 			pt = *(nilfs_cleanerd_protection_period(cleanerd));
 		} else {
-			pt.tv_sec = (oldest > prottime ? oldest - prottime : 0) + 1;
-			pt.tv_usec = 0;
+			__s64 tgt;
+
+			ret = clock_gettime(CLOCK_REALTIME, &curr);
+			if (ret < 0) {
+				syslog(LOG_ERR, "cannot get current time: %m");
+				return -1;
+			}
+			tgt = min_t(__s64, oldest, curr.tv_sec);
+			pt.tv_sec = tgt > prottime ? tgt - prottime + 1 : 1;
+			pt.tv_nsec = 0;
 		}
-		if (timercmp(&pt,
-			     &cleanerd->config.cf_clean_check_interval, <))
+		if (timespeccmp(&pt,
+				&cleanerd->config.cf_clean_check_interval, <))
 			cleanerd->timeout =
 				cleanerd->config.cf_clean_check_interval;
 		else
@@ -870,26 +907,31 @@ static int nilfs_cleanerd_recalc_interval(struct nilfs_cleanerd *cleanerd,
 		return 0;
 	}
 
+	ret = clock_gettime(CLOCK_MONOTONIC, &curr);
+	if (ret < 0) {
+		syslog(LOG_ERR, "cannot get monotonic time: %m");
+		return -1;
+	}
+
 	if (cleanerd->fallback) {
-		timeradd(&curr, &cleanerd->config.cf_retry_interval,
-			 &cleanerd->target);
-		timersub(&cleanerd->target, &curr, &cleanerd->timeout);
+		timespecadd(&curr, &cleanerd->config.cf_retry_interval,
+			    &cleanerd->target);
+		timespecsub(&cleanerd->target, &curr, &cleanerd->timeout);
 		syslog(LOG_DEBUG, "retry interval");
 		return 0;
 	}
 
 	interval = nilfs_cleanerd_cleaning_interval(cleanerd);
-	/* timercmp() does not work for '>=' or '<='. */
+	/* timespeccmp() does not work for '>=' or '<='. */
 	/* curr >= target */
-	if (!timercmp(&curr, &cleanerd->target, <) || cleanerd->no_timeout) {
-		cleanerd->timeout.tv_sec = 0;
-		cleanerd->timeout.tv_usec = 0;
-		timeradd(&curr, interval, &cleanerd->target);
+	if (!timespeccmp(&curr, &cleanerd->target, <) || cleanerd->no_timeout) {
+		timespecclear(&cleanerd->timeout);
+		timespecadd(&curr, interval, &cleanerd->target);
 		syslog(LOG_DEBUG, "adjust interval");
 		return 0;
 	}
-	timersub(&cleanerd->target, &curr, &cleanerd->timeout);
-	timeradd(&cleanerd->target, interval, &cleanerd->target);
+	timespecsub(&cleanerd->target, &curr, &cleanerd->timeout);
+	timespecadd(&cleanerd->target, interval, &cleanerd->target);
 	return 0;
 }
 
@@ -980,7 +1022,7 @@ static int nilfs_cleanerd_cmd_run(struct nilfs_cleanerd *cleanerd,
 			goto error_inval;
 		cleanerd->mm_protection_period.tv_sec =
 			req2->args.protection_period;
-		cleanerd->mm_protection_period.tv_usec = 0;
+		cleanerd->mm_protection_period.tv_nsec = 0;
 	} else {
 		cleanerd->mm_protection_period =
 			cleanerd->config.cf_protection_period;
@@ -1000,8 +1042,8 @@ static int nilfs_cleanerd_cmd_run(struct nilfs_cleanerd *cleanerd,
 	if (req2->args.valid & NILFS_CLEANER_ARG_CLEANING_INTERVAL) {
 		cleanerd->mm_cleaning_interval.tv_sec =
 			req2->args.cleaning_interval;
-		cleanerd->mm_cleaning_interval.tv_usec =
-			req2->args.cleaning_interval_nsec / 1000;
+		cleanerd->mm_cleaning_interval.tv_nsec =
+			req2->args.cleaning_interval_nsec;
 	} else {
 		cleanerd->mm_cleaning_interval =
 			cleanerd->cleaning_interval;
@@ -1201,34 +1243,49 @@ out:
 
 static int nilfs_cleanerd_wait(struct nilfs_cleanerd *cleanerd)
 {
-	struct timeval curr, next;
-	struct timespec abs_timeout;
-	struct timeval *timeout = &cleanerd->timeout;
+	struct pollfd pfd;
 	ssize_t bytes;
+	int ret;
 
-	syslog(LOG_DEBUG, "wait %ld.%06ld",
-	       timeout->tv_sec, timeout->tv_usec);
+	syslog(LOG_DEBUG, "wait %ld.%09ld",
+	       cleanerd->timeout.tv_sec, cleanerd->timeout.tv_nsec);
 
-	if (gettimeofday(&curr, NULL) < 0) {
-		syslog(LOG_ERR, "cannot get current time: %m");
+	memset(&pfd, 0, sizeof(pfd));
+	pfd.fd = cleanerd->recvq;
+	pfd.events = POLLIN;
+
+	ret = ppoll(&pfd, 1, &cleanerd->timeout, NULL);
+	if (ret < 0) {
+		if (errno == EINTR) {
+			syslog(LOG_INFO, "wake up (interrupted)");
+			goto out;
+		}
+		syslog(LOG_ERR, "ppoll failed: %m");
 		return -1;
 	}
-	timeradd(&curr, timeout, &next);
-	timeval_to_timespec(&next, &abs_timeout);
 
-	bytes = mq_timedreceive(cleanerd->recvq, nilfs_cleanerd_msgbuf,
-				sizeof(nilfs_cleanerd_msgbuf), NULL,
-				&abs_timeout);
+	if (!(pfd.revents & POLLIN)) {
+		syslog(LOG_DEBUG, "wake up (timed out)");
+		goto out;
+	}
+	syslog(LOG_DEBUG, "wake up to handle message");
+
+	bytes = mq_receive(cleanerd->recvq, nilfs_cleanerd_msgbuf,
+			   sizeof(nilfs_cleanerd_msgbuf), NULL);
 	if (bytes < 0) {
-		if (errno != ETIMEDOUT && errno != EINTR) {
-			syslog(LOG_ERR, "cannot sleep: %m");
+		if (errno == EINTR || errno == EAGAIN) {
+			syslog(LOG_INFO, "mq_receive aborted: %s",
+			       errno == EINTR ?
+			       "interrupted" : "no message found");
+		} else {
+			syslog(LOG_ERR, "mq_receive failed: %m");
 			return -1;
 		}
-		syslog(LOG_DEBUG, "wake up");
 	} else {
 		nilfs_cleanerd_handle_message(cleanerd, nilfs_cleanerd_msgbuf,
 					      bytes);
 	}
+out:
 	return 0;
 }
 
@@ -1376,11 +1433,11 @@ static void nilfs_cleanerd_progress(struct nilfs_cleanerd *cleanerd, int nsegs)
 
 static int nilfs_cleanerd_clean_segments(struct nilfs_cleanerd *cleanerd,
 					 __u64 *segnums, size_t nsegs,
-					 __u64 protseq, __u64 prottime,
-					 size_t *ndone)
+					 __u64 protseq, size_t *ndone)
 {
 	struct nilfs_reclaim_params params;
 	struct nilfs_reclaim_stat stat;
+	struct timespec *pt;
 	int ret, i, sumsegs;
 
 	params.flags = NILFS_RECLAIM_PARAM_PROTSEQ |
@@ -1390,14 +1447,18 @@ static int nilfs_cleanerd_clean_segments(struct nilfs_cleanerd *cleanerd,
 			nilfs_cleanerd_min_reclaimable_blocks(cleanerd);
 	params.protseq = protseq;
 
-	ret = nilfs_cnoconv_time2cno(cleanerd->cnoconv, prottime,
-				     &params.protcno);
+	pt = nilfs_cleanerd_protection_period(cleanerd);
+
+	ret = nilfs_cnormap_track_back(cleanerd->cnormap, pt->tv_sec,
+				       &params.protcno);
 	if (ret < 0) {
 		syslog(LOG_ERR,
-		       "cannot convert protection time to checkpoint " \
-		       "number: %m");
+		       "cannot get checkpoint number from protection period (%llu): %m",
+		       (unsigned long long)pt->tv_sec);
 		goto out;
 	}
+	syslog(LOG_DEBUG, "got cno %llu from protection period %lu",
+	       (unsigned long long)params.protcno, (unsigned long)pt->tv_sec);
 
 	memset(&stat, 0, sizeof(stat));
 	ret = nilfs_xreclaim_segment(cleanerd->nilfs, segnums, nsegs, 0,
@@ -1468,7 +1529,7 @@ out:
 static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 {
 	struct nilfs_sustat sustat;
-	__u64 prottime = 0, oldest = 0;
+	__s64 prottime = 0, oldest = 0;
 	__u64 segnums[NILFS_CLDCONFIG_NSEGMENTS_PER_CLEAN_MAX];
 	sigset_t sigset;
 	size_t ndone;
@@ -1495,7 +1556,6 @@ static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 	cleanerd->running = 1;
 	cleanerd->fallback = 0;
 	cleanerd->retry_cleaning = 0;
-	nilfs_cnoconv_reset(cleanerd->cnoconv);
 	nilfs_gc_logger = syslog;
 
 	ret = nilfs_cleanerd_init_interval(cleanerd);
@@ -1549,7 +1609,7 @@ static int nilfs_cleanerd_clean_loop(struct nilfs_cleanerd *cleanerd)
 		if (ns > 0) {
 			ret = nilfs_cleanerd_clean_segments(
 				cleanerd, segnums, ns, sustat.ss_prot_seq,
-				prottime, &ndone);
+				&ndone);
 			if (ret < 0)
 				return -1;
 		} else {
@@ -1578,8 +1638,7 @@ sleep:
 int main(int argc, char *argv[])
 {
 	char *progname, *conffile;
-	char canonical[PATH_MAX + 2];
-	const char *dev, *dir;
+	char *dev, *dir;
 	char *endptr;
 	int status, c;
 #ifdef _GNU_SOURCE
@@ -1589,7 +1648,7 @@ int main(int argc, char *argv[])
 	progname = (strrchr(argv[0], '/') != NULL) ?
 		strrchr(argv[0], '/') + 1 : argv[0];
 	conffile = NILFS_CLEANERD_CONFFILE;
-	status = 0;
+	status = EXIT_SUCCESS;
 	protection_period = ULONG_MAX;
 	dev = NULL;
 	dir = NULL;
@@ -1607,85 +1666,84 @@ int main(int argc, char *argv[])
 			break;
 		case 'h':
 			nilfs_cleanerd_usage(progname);
-			exit(0);
+			exit(EXIT_SUCCESS);
 		case 'n':
 			/* ignore nofork option, do nothing */
 			break;
 		case 'p':
 			protection_period = strtoul(optarg, &endptr, 10);
 			if (endptr == optarg || *endptr != '\0') {
-				fprintf(stderr,
-					"%s: invalid protection period: %s\n",
-					progname, optarg);
-				exit(1);
+				errx(EXIT_FAILURE,
+				     "invalid protection period: %s", optarg);
 			} else if (protection_period == ULONG_MAX &&
 				   errno == ERANGE) {
-				fprintf(stderr, "%s: too large period: %s\n",
-					progname, optarg);
-				exit(1);
+				errx(EXIT_FAILURE, "too large period: %s",
+				     optarg);
 			}
 			break;
 		case 'V':
 			nilfs_cleanerd_version(progname);
-			exit(0);
+			exit(EXIT_SUCCESS);
 		default:
 			nilfs_cleanerd_usage(progname);
-			exit(1);
+			exit(EXIT_FAILURE);
 		}
 	}
 
-	if (optind < argc)
-		dev = argv[optind++];
+	if (optind < argc) {
+		const char *path = argv[optind++];
 
-	if (optind < argc)
-		dir = argv[optind++];
-
-	if (dev && myrealpath(dev, canonical, sizeof(canonical))) {
-		dev = strdup(canonical);
-		if (!dev) {
-			fprintf(stderr, "%s: %s\n", progname, strerror(ENOMEM));
-			exit(1);
-		}
+		dev = get_canonical_path(path);
+		if (path && !dev)
+			err(EXIT_FAILURE,
+			    "failed to canonicalize device path %s", path);
 	}
 
-	if (dir && myrealpath(dir, canonical, sizeof(canonical))) {
-		dir = strdup(canonical);
-		if (!dir) {
-			fprintf(stderr, "%s: %s\n", progname, strerror(ENOMEM));
-			exit(1);
+	if (optind < argc) {
+		const char *path = argv[optind++];
+
+		dir = get_canonical_path(path);
+		if (path && !dir) {
+			warn("failed to canonicalize directory path %s", path);
+			status = EXIT_FAILURE;
+			goto out_free;
 		}
 	}
 
 	if (daemonize(0, 0) < 0) {
-		fprintf(stderr, "%s: %s\n", progname, strerror(errno));
-		exit(1);
-	}
-
-	if (oom_adjust() < 0) {
-		fprintf(stderr, "adjusting the OOM killer failed: %m\n");
-		exit(1);
+		warn(NULL);
+		status = EXIT_FAILURE;
+		goto out_free;
 	}
 
 	openlog(progname, LOG_PID, LOG_DAEMON);
 	syslog(LOG_INFO, "start");
 
+	if (oom_adjust() < 0)
+		syslog(LOG_WARNING,
+		       "adjusting the OOM killer failed: %m");
+
 	nilfs_cleanerd = nilfs_cleanerd_create(dev, dir, conffile);
 	if (nilfs_cleanerd == NULL) {
 		syslog(LOG_ERR, "cannot create cleanerd on %s: %m", dev);
-		status = 1;
-		goto out;
+		status = EXIT_FAILURE;
+		goto out_close_log;
 	}
 
 	if (!sigsetjmp(nilfs_cleanerd_env, 1)) {
 		if (nilfs_cleanerd_clean_loop(nilfs_cleanerd) < 0)
-			status = 1;
+			status = EXIT_FAILURE;
 	}
 
 	nilfs_cleanerd_destroy(nilfs_cleanerd);
 
- out:
+out_close_log:
 	syslog(LOG_INFO, "shutdown");
 	closelog();
 
+out_free:
+	free(dir);	/* free(NULL) is just ignored */
+	free(dev);
+
 	exit(status);
 }
