@@ -15,12 +15,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with NILFS; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
- * Written by Hisashi Hifumi <hifumi@osrg.net>,
- *            Amagai Yoshiji <amagai@osrg.net>.
+ * Written by Hisashi Hifumi,
+ *            Amagai Yoshiji.
  * Revised by Ryusuke Konishi <konishi.ryusuke@lab.ntt.co.jp>.
  */
 
@@ -137,7 +133,7 @@ static void parse_options(int argc, char *argv[]);
  *           ^
  *           sb->s_first_data_block
  *
- * Intial layout of ifile and DAT file:
+ * Initial layout of ifile and DAT file:
  *
  * blk +0                  +1       +2
  *  +------------------+--------+---------+
@@ -377,8 +373,9 @@ static unsigned count_ifile_blocks(void)
 
 static unsigned count_sufile_blocks(void)
 {
-	unsigned long sufile_segment_usages_per_block
-		= blocksize / sizeof(struct nilfs_segment_usage);
+	unsigned long sufile_segment_usages_per_block =
+		blocksize / sizeof(struct nilfs_segment_usage);
+
 	return DIV_ROUND_UP(nr_initial_segments +
 			   NILFS_SUFILE_FIRST_SEGMENT_USAGE_OFFSET,
 			   sufile_segment_usages_per_block);
@@ -387,8 +384,9 @@ static unsigned count_sufile_blocks(void)
 static unsigned count_cpfile_blocks(void)
 {
 	const unsigned nr_initial_checkpoints = 1;
-	unsigned long cpfile_checkpoints_per_block
-		= blocksize / sizeof(struct nilfs_checkpoint);
+	unsigned long cpfile_checkpoints_per_block =
+		blocksize / sizeof(struct nilfs_checkpoint);
+
 	return DIV_ROUND_UP(nr_initial_checkpoints +
 			   NILFS_CPFILE_FIRST_CHECKPOINT_OFFSET
 			   - 1 /* checkpoint number begins from 1 */,
@@ -509,8 +507,8 @@ static void init_disk_layout(struct nilfs_disk_info *di, int fd,
 	if (di->nsegments < min_nsegments)
 		perr("Error: too small device.\n"
 		     "       device size=%llu bytes, required size=%llu bytes.\n"
-		     "       Please enlarge the device, "
-		     "or shorten segments with -B option.", dev_size,
+		     "       Please enlarge the device, or shorten segments with -B option.",
+		     dev_size,
 		     (unsigned long long)segment_size * min_nsegments);
 	di->nseginfo = 0;
 }
@@ -532,12 +530,12 @@ static struct nilfs_segment_info *new_segment(struct nilfs_disk_info *di)
 
 static void fix_disk_layout(struct nilfs_disk_info *di)
 {
+	struct nilfs_segment_info *si;
 	int i, j;
 
 	di->nblocks_used = 0;
 	di->nblocks_to_write = di->first_segment_block;
-	for (i = 0; i < di->nseginfo; i++) {
-		struct nilfs_segment_info *si = &di->seginfo[i];
+	for (i = 0, si = di->seginfo; i < di->nseginfo; i++, si++) {
 		blocknr_t blocknr = si->start + si->nblk_sum;
 
 		si->nblocks += si->nblk_sum + 1 /* summary and super root */;
@@ -723,8 +721,7 @@ static void check_mount(int fd, const char *device)
 		if (strncmp(strtok(line, " "), device, strlen(device)) == 0) {
 			fclose(fp);
 			close(fd);
-			perr("Error: %s is currently mounted. " \
-			     "You cannot make a filesystem on this device.",
+			perr("Error: %s is currently mounted. You cannot make a filesystem on this device.",
 			     device);
 		}
 	}
@@ -765,18 +762,16 @@ static void check_safety_of_device_overwrite(int fd, const char *device)
 
 			if (!blkid_probe_lookup_value(pr, "TYPE",
 							&type, NULL)) {
-				pinfo("WARNING: Device %s appears to contain" \
-				      " an existing %s superblock.",
+				pinfo("WARNING: Device %s appears to contain an existing %s superblock.",
 				      device, type);
 			} else if (!blkid_probe_lookup_value(pr, "PTTYPE",
 								&type, NULL)) {
-				pinfo("WARNING: Device %s appears to contain" \
-				      " an partition table (%s).",
+				pinfo("WARNING: Device %s appears to contain an partition table (%s).",
 				      device, type);
 			} else {
 				if (quiet == 0) {
-					pinfo("Device %s appears to contain" \
-					      " something weird.", device);
+					pinfo("Device %s appears to contain something weird.",
+					      device);
 				}
 				goto end_check;
 			}
@@ -819,7 +814,6 @@ abort_format:
 		blkid_free_probe(pr);
 	close(fd);
 	perr("Abort format of device %s", device);
-	return;
 }
 #endif /* HAVE_LIBBLKID */
 
@@ -853,8 +847,8 @@ static void init_disk_buffer(long max_blocks)
 static void *map_disk_buffer(blocknr_t blocknr, int clear_flag)
 {
 	if (blocknr >= disk_buffer_size)
-		perr("Internal error: illegal disk buffer access " \
-		     "(blocknr=%llu)", blocknr);
+		perr("Internal error: illegal disk buffer access (blocknr=%llu)",
+		     blocknr);
 
 	if (!disk_buffer[blocknr]) {
 		if (posix_memalign(&disk_buffer[blocknr], blocksize,
@@ -939,8 +933,7 @@ static int erase_disk(int fd, struct nilfs_disk_info *di)
 		ret = nilfs_mkfs_discard_range(fd, start, end - start);
 		if (!ret && nilfs_mkfs_discard_zeroes_data(fd)) {
 			if (verbose)
-				pinfo("Discard succeeded and will return 0s " \
-				      " - skip wiping");
+				pinfo("Discard succeeded and will return 0s  - skip wiping");
 			goto out;
 		}
 	}
@@ -1160,10 +1153,9 @@ static void parse_options(int argc, char *argv[])
 static void usage(void)
 {
 	fprintf(stderr,
-		"Usage: %s [-b block-size] [-B blocks-per-segment] [-c] " \
-		"[-f]\n" \
-		"       [-L volume-label] [-m reserved-segments-percentage]\n" \
-		"       [-O feature[,...]]\n" \
+		"Usage: %s [-b block-size] [-B blocks-per-segment] [-c] [-f]\n"
+		"       [-L volume-label] [-m reserved-segments-percentage]\n"
+		"       [-O feature[,...]]\n"
 		"       [-hnqvKV] device\n",
 		progname);
 }
@@ -1374,8 +1366,8 @@ static void update_blocknr(struct nilfs_file_info *fi,
 static void prepare_blockgrouped_file(blocknr_t blocknr)
 {
 	struct nilfs_palloc_group_desc *desc;
-	const unsigned group_descs_per_block
-		= blocksize / sizeof(struct nilfs_palloc_group_desc);
+	const unsigned group_descs_per_block =
+		blocksize / sizeof(struct nilfs_palloc_group_desc);
 	int i;
 
 	for (i = 0, desc = map_disk_buffer(blocknr, 1);
@@ -1388,7 +1380,7 @@ static inline void
 alloc_blockgrouped_file_entry(blocknr_t blocknr, unsigned long nr)
 {
 	struct nilfs_palloc_group_desc *desc = map_disk_buffer(blocknr, 1);
-					/* allways use the first group */
+					/* always use the first group */
 	void *bitmap = map_disk_buffer(blocknr + 1, 1);
 
 	if (nilfs_test_bit(nr, bitmap))
@@ -1402,8 +1394,8 @@ static void prepare_ifile(void)
 {
 	struct nilfs_file_info *fi = nilfs.files[NILFS_IFILE_INO];
 	struct nilfs_inode *raw_inode;
-	const unsigned entries_per_block
-		= blocksize / sizeof(struct nilfs_inode);
+	const unsigned entries_per_block =
+		blocksize / sizeof(struct nilfs_inode);
 	blocknr_t entry_block;
 	blocknr_t blocknr = fi->start;
 	int i;
@@ -1440,8 +1432,8 @@ static void reserve_ifile_inode(ino_t ino)
 static void prepare_cpfile(void)
 {
 	struct nilfs_file_info *fi = nilfs.files[NILFS_CPFILE_INO];
-	const unsigned entries_per_block
-		= blocksize / sizeof(struct nilfs_checkpoint);
+	const unsigned entries_per_block =
+		blocksize / sizeof(struct nilfs_checkpoint);
 	blocknr_t blocknr = fi->start;
 	blocknr_t entry_block = blocknr;
 	struct nilfs_cpfile_header *header;
@@ -1499,8 +1491,8 @@ static void commit_cpfile(void)
 static void prepare_sufile(void)
 {
 	struct nilfs_file_info *fi = nilfs.files[NILFS_SUFILE_INO];
-	const unsigned entries_per_block
-		= blocksize / sizeof(struct nilfs_segment_usage);
+	const unsigned entries_per_block =
+		blocksize / sizeof(struct nilfs_segment_usage);
 	blocknr_t blocknr = fi->start;
 	blocknr_t entry_block = blocknr;
 	struct nilfs_sufile_header *header;
@@ -1538,8 +1530,8 @@ static void prepare_sufile(void)
 static void commit_sufile(void)
 {
 	struct nilfs_file_info *fi = nilfs.files[NILFS_SUFILE_INO];
-	const unsigned entries_per_block
-		= blocksize / sizeof(struct nilfs_segment_usage);
+	const unsigned entries_per_block =
+		blocksize / sizeof(struct nilfs_segment_usage);
 	struct nilfs_segment_usage *su;
 	unsigned segnum = fi->start / nilfs.diskinfo->blocks_per_segment;
 	blocknr_t blocknr = fi->start +
@@ -1557,8 +1549,8 @@ static void prepare_dat(void)
 {
 	struct nilfs_file_info *fi = nilfs.files[NILFS_DAT_INO];
 	struct nilfs_dat_entry *entry;
-	const unsigned entries_per_block
-		= blocksize / sizeof(struct nilfs_dat_entry);
+	const unsigned entries_per_block =
+		blocksize / sizeof(struct nilfs_dat_entry);
 	blocknr_t entry_block;
 	int i, vblocknr = 0;
 	blocknr_t blocknr = fi->start;
@@ -1584,8 +1576,8 @@ static void prepare_dat(void)
 static blocknr_t assign_vblocknr(blocknr_t blocknr)
 {
 	struct nilfs_file_info *fi = nilfs.files[NILFS_DAT_INO];
-	const unsigned entries_per_block
-		= blocksize / sizeof(struct nilfs_dat_entry);
+	const unsigned entries_per_block =
+		blocksize / sizeof(struct nilfs_dat_entry);
 	struct nilfs_dat_entry *entry;
 	blocknr_t vblocknr = nilfs.vblocknr++;
 	blocknr_t entry_block = fi->start + group_desc_blocks_per_group
